@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -33,19 +34,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -69,15 +71,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -88,7 +91,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: FingerPrintResults.h 21904 2011-01-21 00:04:16Z fyodor $ */
+/* $Id: FingerPrintResults.h 28192 2012-03-01 06:53:35Z fyodor $ */
 
 #ifndef FINGERPRINTRESULTS_H
 #define FINGERPRINTRESULTS_H
@@ -97,6 +100,7 @@ class FingerPrintResults;
 
 #include "nmap.h"
 #include "global_structures.h"
+#include "FPEngine.h"
 
 /* Maximum number of results allowed in one of these things ... */
 #define MAX_FP_RESULTS 36
@@ -112,20 +116,20 @@ struct OS_Classification_Results {
 class FingerPrintResults {
  public: /* For now ... a lot of the data members should be made private */
   FingerPrintResults();
-  ~FingerPrintResults();
+  virtual ~FingerPrintResults();
 
   double accuracy[MAX_FP_RESULTS]; /* Percentage of match (1.0 == perfect 
-				      match) in same order as pritns[] below */
-  FingerPrint *prints[MAX_FP_RESULTS]; /* ptrs to matching references -- 
+				      match) in same order as matches[] below */
+  FingerMatch *matches[MAX_FP_RESULTS]; /* ptrs to matching references -- 
 					      highest accuracy matches first */
-  int num_perfect_matches; /* Number of 1.0 accuracy matches in prints[] */
-  int num_matches; /* Total number of matches in prints[] */
+  int num_perfect_matches; /* Number of 1.0 accuracy matches in matches[] */
+  int num_matches; /* Total number of matches in matches[] */
   int overall_results; /* OSSCAN_TOOMANYMATCHES, OSSCAN_NOMATCHES, 
 			  OSSCAN_SUCCESS, etc */
 
   /* Ensures that the results are available and then returns them.
    You should only call this AFTER all matching has been completed
-   (because results are cached and won't change if new prints[] are
+   (because results are cached and won't change if new matches[] are
    added.)  All OS Classes in the results will be unique, and if there
    are any perfect (accuracy 1.0) matches, only those will be
    returned */
@@ -147,14 +151,13 @@ class FingerPrintResults {
      consistancy standpoint. */
   double maxTimingRatio;
 
-  FingerPrint **FPs; /* Fingerprint data obtained from host */
-  int numFPs;
-
 /* If the fingerprint is of potentially poor quality, we don't want to
    print it and ask the user to submit it.  In that case, the reason
    for skipping the FP is returned as a static string.  If the FP is
    great and should be printed, NULL is returned. */
-  const char *OmitSubmissionFP();
+  virtual const char *OmitSubmissionFP();
+
+  virtual const char *merge_fpr(const Target *currenths, bool isGoodFP, bool wrapit) const = 0;
 
  private:
   bool isClassified; // Whether populateClassification() has been called
@@ -165,4 +168,27 @@ class FingerPrintResults {
   struct OS_Classification_Results OSR;
 };
 
+class FingerPrintResultsIPv4 : public FingerPrintResults {
+public:
+  FingerPrint **FPs; /* Fingerprint data obtained from host */
+  int numFPs;
+
+  FingerPrintResultsIPv4();
+  virtual ~FingerPrintResultsIPv4();
+  const char *merge_fpr(const Target *currenths, bool isGoodFP, bool wrapit) const;
+};
+
+class FingerPrintResultsIPv6 : public FingerPrintResults {
+public:
+  FPResponse *fp_responses[NUM_FP_PROBES_IPv6];
+  struct timeval begin_time;
+  /* The flow label we set in our sent packets, for calculating offsets later. */
+  unsigned int flow_label;
+
+  FingerPrintResultsIPv6();
+  virtual ~FingerPrintResultsIPv6();
+  const char *OmitSubmissionFP();
+  const char *merge_fpr(const Target *currenths, bool isGoodFP, bool wrapit) const;
+};
+
 #endif /* FINGERPRINTRESULTS_H */
