@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -33,19 +34,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -69,15 +71,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -88,7 +91,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 22066 2011-01-27 21:49:15Z david $ */
+/* $Id: NmapOps.cc 28544 2012-05-08 05:49:51Z david $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -107,8 +110,53 @@ NmapOps::NmapOps() {
 }
 
 NmapOps::~NmapOps() {
-  if (datadir) free(datadir);
-  if (xsl_stylesheet) free(xsl_stylesheet);
+  if (xsl_stylesheet) {
+    free(xsl_stylesheet);
+    xsl_stylesheet = NULL;
+  }
+
+  if (reference_FPs) {
+    delete reference_FPs;
+    reference_FPs = NULL;
+  }
+
+  if (dns_servers) {
+    free(dns_servers);
+    dns_servers = NULL;
+  }
+  if (extra_payload) {
+    free(extra_payload);
+    extra_payload = NULL;
+  }
+  if (ipoptions) {
+    free(ipoptions);
+    ipoptions = NULL;
+  }
+  if (portlist) {
+    free(portlist);
+    portlist = NULL;
+  }
+  if (exclude_spec) {
+    free(exclude_spec);
+    exclude_spec = NULL;
+  }
+  if (idleProxy) {
+    free(idleProxy);
+    idleProxy = NULL;
+  }
+  if (datadir) {
+    free(datadir);
+    datadir = NULL;
+  }
+
+#ifndef NOLUA
+  if (scriptversion || script)
+    close_nse();
+  if (scriptargs) {
+    free(scriptargs);
+    scriptargs = NULL;
+  }
+#endif
 }
 
 void NmapOps::ReInit() {
@@ -131,6 +179,14 @@ int NmapOps::SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
   return 0;
 }
 
+/* Returns a const pointer to the source address if set, or NULL if unset. */
+const struct sockaddr_storage *NmapOps::SourceSockAddr() const {
+  if (sourcesock.ss_family == AF_UNSPEC)
+    return NULL;
+  else
+    return &sourcesock;
+}
+
 /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
 void NmapOps::setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
@@ -139,31 +195,15 @@ void NmapOps::setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
   sourcesocklen = ss_len;
 }
 
-struct in_addr NmapOps::v4source() {
- const struct in_addr *addy = v4sourceip();
-  struct in_addr in;
-  if (addy) return *addy;
-  in.s_addr = 0;
-  return in;
-}
-
-const struct in_addr *NmapOps::v4sourceip() {
-   struct sockaddr_in *sin = (struct sockaddr_in *) &sourcesock;
-  if (sin->sin_family == AF_INET) {
-    return &(sin->sin_addr);
-  }
-  return NULL;
-}
-
-// Number of milliseconds since getStartTime().  The current time is an
+// Number of seconds since getStartTime().  The current time is an
 // optional argument to avoid an extra gettimeofday() call.
-int NmapOps::TimeSinceStartMS(const struct timeval *now) {
+float NmapOps::TimeSinceStart(const struct timeval *now) {
   struct timeval tv;
   if (!now)
     gettimeofday(&tv, NULL);
   else tv = *now;
 
-  return TIMEVAL_MSEC_SUBTRACT(tv, start_time);
+  return TIMEVAL_FSEC_SUBTRACT(tv, start_time);
 }
 
 // Convert a filename to a file:// URL. The return value must be freed.
@@ -195,8 +235,6 @@ static char *filename_to_url(const char *filename) {
 }
 
 void NmapOps::Initialize() {
-  char tmpxsl[MAXPATHLEN];
-
   setaf(AF_INET);
 #if defined WIN32 || defined __amigaos__
   isr00t = 1;
@@ -215,6 +253,7 @@ void NmapOps::Initialize() {
   max_packet_send_rate = 0.0; /* Unset. */
   stats_interval = 0.0; /* Unset. */
   randomize_hosts = 0;
+  randomize_ports = 1;
   sendpref = PACKET_SEND_NOPREF;
   spoofsource = 0;
   fastscan = 0;
@@ -256,7 +295,7 @@ void NmapOps::Initialize() {
   version_intensity = 7;
   pingtype = PINGTYPE_UNKNOWN;
   listscan = allowall = ackscan = bouncescan = connectscan = 0;
-  rpcscan = nullscan = xmasscan = fragscan = synscan = windowscan = 0;
+  nullscan = xmasscan = fragscan = synscan = windowscan = 0;
   maimonscan = idlescan = finscan = udpscan = ipprotscan = 0;
   noportscan = noresolve = 0;
   sctpinitscan = 0;
@@ -272,23 +311,13 @@ void NmapOps::Initialize() {
   adler32 = false;
   if (datadir) free(datadir);
   datadir = NULL;
+  xsl_stylesheet_set = false;
   if (xsl_stylesheet) free(xsl_stylesheet);
-  if (nmap_fetchfile(tmpxsl, sizeof(tmpxsl), "nmap.xsl") == 1) {
-    xsl_stylesheet = filename_to_url(tmpxsl);
-  } else {
-#if WIN32
-    /* Use a relative URL on Windows if nmap_fetchfile failed. It won't work,
-       but it gives a clue that there is an nmap.xsl somewhere. */
-    Strncpy(tmpxsl, "nmap.xsl", sizeof(tmpxsl));
-    xsl_stylesheet = strdup(tmpxsl);
-#else
-    Snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
-    xsl_stylesheet = filename_to_url(tmpxsl);
-#endif
-  }
+  xsl_stylesheet = NULL;
   spoof_mac_set = false;
   mass_dns = true;
   log_errors = false;
+  deprecated_xml_osclass = false;
   resolve_all = 0;
   dns_servers = NULL;
   numhosts_scanned = 0;
@@ -313,6 +342,12 @@ void NmapOps::Initialize() {
 #endif
   memset(&sourcesock, 0, sizeof(sourcesock));
   sourcesocklen = 0;
+  excludefd = NULL;
+  exclude_spec = NULL;
+  quashargv = 0;
+  inputfd = NULL;
+  idleProxy = NULL;
+  portlist = NULL;
 }
 
 bool NmapOps::SCTPScan() {
@@ -332,9 +367,9 @@ bool NmapOps::RawScan() {
     return true;
   if (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS|PINGTYPE_TCP_USE_ACK|PINGTYPE_UDP|PINGTYPE_SCTP_INIT))
     return true;
-  /* A SYN scan will only generate raw packets if nmap is running as root and is
-     not issuing IPv6 packets.  Otherwise, it becomes a connect scan. */
-  if ((pingtype & PINGTYPE_TCP_USE_SYN) && (af() == AF_INET) && isr00t)
+  /* A SYN scan will only generate raw packets if nmap is running as root.
+     Otherwise, it becomes a connect scan. */
+  if ((pingtype & PINGTYPE_TCP_USE_SYN) && isr00t)
     return true;
 
    return false; 
@@ -344,7 +379,7 @@ bool NmapOps::RawScan() {
 void NmapOps::ValidateOptions() {
 	const char *privreq = "root privileges.";
 #ifdef WIN32
-	if (!o.have_pcap)
+	if (!have_pcap)
 		privreq = "WinPcap version 3.1 or higher and\n\
 iphlpapi.dll.  You seem to be missing one or both of these.  Winpcap is\n\
 available from http://www.winpcap.org.  iphlpapi.dll comes with Win98 and\n\
@@ -359,7 +394,7 @@ dialog where you can start NPF if you have administrator privileges.";
 
   /* Insure that at least one scantype is selected */
   if (!noportscan && !(TCPScan() || UDPScan() || SCTPScan() || ipprotscan)) {
-    if (isr00t && af() == AF_INET)
+    if (isr00t)
       synscan++;
     else connectscan++;
     //    if (verbose) error("No TCP, UDP, SCTP or ICMP scantype specified, assuming %s scan. Use -sn if you really don't want to portscan (and just want to see what hosts are up).", synscan? "SYN Stealth" : "vanilla tcp connect()");
@@ -382,8 +417,8 @@ dialog where you can start NPF if you have administrator privileges.";
     error("WARNING:  -S will only affect the source address used in a connect() scan if you specify one of your own addresses.  Use -sS or another raw scan if you want to completely spoof your source address, but then you need to know what you're doing to obtain meaningful results.");
   }
 
- if ((pingtype & PINGTYPE_UDP) && (!isr00t || af() != AF_INET)) {
-   fatal("Sorry, UDP Ping (-PU) only works if you are root (because we need to read raw responses off the wire) and only for IPv4 (cause fyodor is too lazy right now to add IPv6 support and nobody has sent a patch)");
+ if ((pingtype & PINGTYPE_UDP) && (!isr00t)) {
+   fatal("Sorry, UDP Ping (-PU) only works if you are root (because we need to read raw responses off the wire)");
  }
 
  if ((pingtype & PINGTYPE_SCTP_INIT) && (!isr00t || af() != AF_INET)) {
@@ -402,8 +437,8 @@ dialog where you can start NPF if you have administrator privileges.";
    fatal("-sL and -sn (skip port scan) are not valid with any other scan types");
  }
 
- if (af() == AF_INET6 && (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))) {
-   fatal("ICMP Echo, Timestamp and Address Mask pings are only valid for IPv4.");
+ if (af() == AF_INET6 && (pingtype & (PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))) {
+   fatal("ICMP Timestamp and Address Mask pings are only valid for IPv4.");
  }
 
  if (sendpref == PACKET_SEND_NOPREF) {
@@ -434,10 +469,6 @@ dialog where you can start NPF if you have administrator privileges.";
   }
   
   
-  if (numdecoys > 0 && rpcscan) {
-    error("WARNING:  RPC scan currently does not make use of decoys so don't count on that protection");
-  }
-  
   if (bouncescan && pingtype != PINGTYPE_NONE) 
     log_write(LOG_STDOUT, "Hint: if your bounce scan target hosts aren't reachable from here, remember to use -Pn so we don't try and ping them prior to the scan\n");
   
@@ -499,14 +530,10 @@ dialog where you can start NPF if you have administrator privileges.";
     fatal("--min-rate=%g must be less than or equal to --max-rate=%g", min_packet_send_rate, max_packet_send_rate);
   }
   
-  if (af() == AF_INET6 && (generate_random_ips|numdecoys|osscan|bouncescan|fragscan|ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan|sctpinitscan|sctpcookieechoscan)) {
-    fatal("Sorry -- IPv6 support is currently only available for connect() scan (-sT), ping scan (-sn), and list scan (-sL).  OS detection, random targets and decoys are also not supported with IPv6.  Further support is under consideration.");
+  if (af() == AF_INET6 && (generate_random_ips|numdecoys|bouncescan|fragscan|idlescan)) {
+    fatal("Sorry -- IPv6 support is currently only available for TCP, UDP, and SCTP port scans and list scan (-sL).  OS detection, random targets and decoys are also not supported with IPv6.  Further support is under consideration.");
   }
 
-  /* Prevent performance values from getting out of whack */
-  if (min_parallelism > max_parallelism)
-    max_parallelism = min_parallelism;
-
   if(ipoptions && osscan)
     error("WARNING: Ip options are NOT used while OS scanning!");
 
@@ -576,6 +603,33 @@ void NmapOps::setMaxHostGroupSz(unsigned int sz) {
 void NmapOps::setXSLStyleSheet(const char *xslname) {
   if (xsl_stylesheet) free(xsl_stylesheet);
   xsl_stylesheet = xslname? strdup(xslname) : NULL;
+  xsl_stylesheet_set = true;
+}
+
+/* Returns the full path or URL that should be printed in the XML
+   output xml-stylesheet element.  Returns NULL if the whole element
+   should be skipped */
+char *NmapOps::XSLStyleSheet() {
+  char tmpxsl[MAXPATHLEN];
+
+  if (xsl_stylesheet_set)
+    return xsl_stylesheet;
+
+  if (nmap_fetchfile(tmpxsl, sizeof(tmpxsl), "nmap.xsl") == 1) {
+    xsl_stylesheet = filename_to_url(tmpxsl);
+  } else {
+#if WIN32
+    /* Use a relative URL on Windows if nmap_fetchfile failed. It won't work,
+       but it gives a clue that there is an nmap.xsl somewhere. */
+    Strncpy(tmpxsl, "nmap.xsl", sizeof(tmpxsl));
+    xsl_stylesheet = strdup(tmpxsl);
+#else
+    Snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
+    xsl_stylesheet = filename_to_url(tmpxsl);
+#endif
+  }
+
+  return xsl_stylesheet;
 }
 
 void NmapOps::setSpoofMACAddress(u8 *mac_data) {
