@@ -2,7 +2,7 @@
  * ncat_connect.c -- Ncat connect mode.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -12,11 +12,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -30,19 +31,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -66,15 +68,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -85,7 +88,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_connect.c 21905 2011-01-21 00:04:51Z fyodor $ */
+/* $Id: ncat_connect.c 28192 2012-03-01 06:53:35Z fyodor $ */
 
 #include "base64.h"
 #include "nsock.h"
@@ -148,10 +151,10 @@ static int verify_callback(int ok, X509_STORE_CTX *store)
         char digest_buf[SHA1_STRING_LENGTH + 1];
 
         loguser("Subject: ");
-        X509_NAME_print_ex_fp(stderr, X509_get_subject_name(cert), 0, XN_FLAG_ONELINE);
+        X509_NAME_print_ex_fp(stderr, X509_get_subject_name(cert), 0, XN_FLAG_COMPAT);
         loguser_noprefix("\n");
         loguser("Issuer: ");
-        X509_NAME_print_ex_fp(stderr, X509_get_issuer_name(cert), 0, XN_FLAG_ONELINE);
+        X509_NAME_print_ex_fp(stderr, X509_get_issuer_name(cert), 0, XN_FLAG_COMPAT);
         loguser_noprefix("\n");
 
         assert(ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf)) != NULL);
@@ -186,7 +189,7 @@ static void set_ssl_ctx_options(SSL_CTX *ctx)
             logdebug("Not doing certificate verification.\n");
     }
 
-    if (o.sslcert != NULL && o.sslkey != NULL){
+    if (o.sslcert != NULL && o.sslkey != NULL) {
       if (SSL_CTX_use_certificate_file(ctx, o.sslcert, SSL_FILETYPE_PEM) != 1)
             bye("SSL_CTX_use_certificate_file(): %s.", ERR_error_string(ERR_get_error(), NULL));
       if (SSL_CTX_use_PrivateKey_file(ctx, o.sslkey, SSL_FILETYPE_PEM) != 1)
@@ -457,7 +460,6 @@ bail:
 
 int ncat_connect(void) {
     nsock_pool mypool;
-    nsock_event_id ev;
     int rc;
 
     /* Create an nsock pool */
@@ -486,7 +488,7 @@ int ncat_connect(void) {
             bye("Failed to set hostname on iod.");
 
         if (srcaddr.storage.ss_family != AF_UNSPEC)
-            nsi_set_localaddr(cs.sock_nsi, &srcaddr.storage, srcaddrlen);
+            nsi_set_localaddr(cs.sock_nsi, &srcaddr.storage, sizeof(srcaddr.storage));
 
         if (o.numsrcrtes) {
             unsigned char *ipopts = NULL;
@@ -501,45 +503,45 @@ int ncat_connect(void) {
         }
 
         if (o.udp) {
-            ev = nsock_connect_udp(mypool, cs.sock_nsi, connect_handler,
-                                   NULL, &targetss.sockaddr, targetsslen,
-                                   inet_port(&targetss));
+            nsock_connect_udp(mypool, cs.sock_nsi, connect_handler,
+                              NULL, &targetss.sockaddr, targetsslen,
+                              inet_port(&targetss));
         }
 #ifdef HAVE_OPENSSL
         else if (o.sctp && o.ssl) {
-            ev = nsock_connect_ssl(mypool, cs.sock_nsi, connect_handler,
-                                   o.conntimeout, NULL,
-                                   &targetss.sockaddr, targetsslen,
-                                   IPPROTO_SCTP, inet_port(&targetss),
-                                   NULL);
+            nsock_connect_ssl(mypool, cs.sock_nsi, connect_handler,
+                              o.conntimeout, NULL,
+                              &targetss.sockaddr, targetsslen,
+                              IPPROTO_SCTP, inet_port(&targetss),
+                              NULL);
         }
 #endif
         else if (o.sctp) {
-            ev = nsock_connect_sctp(mypool, cs.sock_nsi, connect_handler,
-                                   o.conntimeout, NULL,
-                                   &targetss.sockaddr, targetsslen,
-                                   inet_port(&targetss));
+            nsock_connect_sctp(mypool, cs.sock_nsi, connect_handler,
+                              o.conntimeout, NULL,
+                              &targetss.sockaddr, targetsslen,
+                              inet_port(&targetss));
         }
 #ifdef HAVE_OPENSSL
         else if (o.ssl) {
-            ev = nsock_connect_ssl(mypool, cs.sock_nsi, connect_handler,
-                                   o.conntimeout, NULL,
-                                   &targetss.sockaddr, targetsslen,
-                                   IPPROTO_TCP, inet_port(&targetss),
-                                   NULL);
+            nsock_connect_ssl(mypool, cs.sock_nsi, connect_handler,
+                              o.conntimeout, NULL,
+                              &targetss.sockaddr, targetsslen,
+                              IPPROTO_TCP, inet_port(&targetss),
+                              NULL);
         }
 #endif
         else {
-            ev = nsock_connect_tcp(mypool, cs.sock_nsi, connect_handler,
-                                   o.conntimeout, NULL,
-                                   &targetss.sockaddr, targetsslen,
-                                   inet_port(&targetss));
+            nsock_connect_tcp(mypool, cs.sock_nsi, connect_handler,
+                              o.conntimeout, NULL,
+                              &targetss.sockaddr, targetsslen,
+                              inet_port(&targetss));
         }
     } else {
         /* A proxy connection. */
         static int connect_socket;
         int len;
-        char* line;
+        char *line;
         size_t n;
 
         if (httpconnect.storage.ss_family != AF_UNSPEC) {
