@@ -19,6 +19,7 @@ import getopt
 import sys
 import time
 import xml.sax
+import xml.sax.saxutils
 import xml.dom.minidom
 
 verbose = False
@@ -30,19 +31,17 @@ class Scan(object):
     container for a list of hosts. It also has utility methods to load itself
     from an Nmap XML file."""
     def __init__(self):
+        self.scanner = None
         self.version = None
+        self.args = None
         self.start_date = None
         self.end_date = None
         self.hosts = []
         self.pre_script_results = []
         self.post_script_results = []
 
-    def find_host(self, id):
-        for host in self.hosts:
-            if host.get_id() == id:
-                return host
-        else:
-            return None
+    def sort_hosts(self):
+        self.hosts.sort(key = lambda h: h.get_id())
 
     def load(self, f):
         """Load a scan from the Nmap XML in the file-like object f."""
@@ -59,6 +58,37 @@ class Scan(object):
         finally:
             f.close()
 
+    def write_nmaprun_open(self, writer):
+        attrs = {}
+        if self.scanner is not None:
+            attrs[u"scanner"] = self.scanner
+        if self.args is not None:
+            attrs[u"args"] = self.args
+        if self.start_date is not None:
+            attrs[u"start"] = "%d" % time.mktime(self.start_date.timetuple())
+            attrs[u"startstr"] = self.start_date.strftime("%a %b %d %H:%M:%S %Y")
+        if self.version is not None:
+            attrs[u"version"] = self.version
+        writer.startElement(u"nmaprun", attrs)
+
+    def write_nmaprun_close(self, writer):
+        writer.endElement(u"nmaprun")
+
+    def nmaprun_to_dom_fragment(self, document):
+        frag = document.createDocumentFragment()
+        elem = document.createElement(u"nmaprun")
+        if self.scanner is not None:
+            elem.setAttribute(u"scanner", self.scanner)
+        if self.args is not None:
+            elem.setAttribute(u"args", self.args)
+        if self.start_date is not None:
+            elem.setAttribute(u"start", "%d" % time.mktime(self.start_date.timetuple()))
+            elem.setAttribute(u"startstr", self.start_date.strftime("%a %b %d %H:%M:%S %Y"))
+        if self.version is not None:
+            elem.setAttribute(u"version", self.version)
+        frag.appendChild(elem)
+        return frag
+
 class Host(object):
     """A single host, with a state, addresses, host names, a dict mapping port
     specs to Ports, and a list of OS matches. Host states are strings, or None
@@ -227,7 +257,7 @@ class Address(object):
         return frag
 
 # The sort_key method in the Address subclasses determines the order in which
-# addresses are displayed. We do IPv4, then IPV6, then MAC.
+# addresses are displayed. We do IPv4, then IPv6, then MAC.
 
 class IPv4Address(Address):
     type = property(lambda self: u"ipv4")
@@ -383,11 +413,17 @@ class ScriptResult(object):
 
 def format_banner(scan):
     """Format a startup banner more or less like Nmap does."""
-    parts = [u"Nmap"]
+    scanner = u"Nmap"
+    if scan.scanner is not None and scan.scanner != u"nmap":
+        scanner = scan.scanner
+    parts = [ scanner ]
     if scan.version is not None:
         parts.append(scan.version)
+    parts.append(u"scan")
     if scan.start_date is not None:
-        parts.append(u"at %s" % scan.start_date.strftime("%Y-%m-%d %H:%M"))
+        parts.append(u"initiated %s" % scan.start_date.strftime("%a %b %d %H:%M:%S %Y"))
+    if scan.args is not None:
+        parts.append(u"as: %s" % scan.args)
     return u" ".join(parts)
 
 def print_script_result_diffs_text(title, script_results_a, script_results_b,
@@ -426,94 +462,152 @@ def script_result_diffs_to_dom_fragment(elem, script_results_a,
             elem.appendChild(sr_diff.to_dom_fragment(document))
         return elem
 
+def host_pairs(a, b):
+    """Take hosts lists a and b, which must be sorted by id, and return pairs.
+    When the heads of both lists have the same ids, they are returned together.
+    Otherwise the one with the smaller id is returned, with an empty host as its
+    counterpart, and the one with the higher id will remain in its list for a
+    later iteration."""
+    i = 0
+    j = 0
+    while i < len(a) and j < len(b):
+        if a[i].get_id() < b[j].get_id():
+            yield a[i], Host()
+            i += 1
+        elif a[i].get_id() > b[j].get_id():
+            yield Host(), b[j]
+            j += 1
+        else:
+            yield a[i], b[j]
+            i += 1
+            j += 1
+    while i < len(a):
+        yield a[i], Host()
+        i += 1
+    while j < len(b):
+        yield Host(), b[j]
+        j += 1
+
 class ScanDiff(object):
-    """A complete diff of two scans. It is a container for two scans and a dict
-    mapping hosts to HostDiffs."""
-    def __init__(self, scan_a, scan_b):
+    """An abtract class for different diff output types. Subclasses must define
+    various output methods."""
+    def __init__(self, scan_a, scan_b, f = sys.stdout):
         """Create a ScanDiff from the "before" scan_a and the "after" scan_b."""
         self.scan_a = scan_a
         self.scan_b = scan_b
-        self.hosts = []
-        self.pre_script_result_diffs = []
-        self.post_script_result_diffs = []
-        self.host_diffs = {}
+        self.f = f
 
-        self.diff()
+    def output(self):
+        self.scan_a.sort_hosts()
+        self.scan_b.sort_hosts()
 
-    def diff(self):
-        a_ids = [h.get_id() for h in self.scan_a.hosts]
-        b_ids = [h.get_id() for h in self.scan_b.hosts]
-        for id in sorted(set(a_ids).union(set(b_ids))):
-            # Currently we never consider diffing hosts with a different id
-            # (address or host name), which could lead to better diffs.
-            host_a = self.scan_a.find_host(id)
-            host_b = self.scan_b.find_host(id)
-            h_diff = HostDiff(host_a or Host(), host_b or Host())
+        self.output_beginning()
+
+        pre_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.pre_script_results, self.scan_b.pre_script_results)
+        self.output_pre_scripts(pre_script_result_diffs)
+
+        cost = 0
+        # Currently we never consider diffing hosts with a different id
+        # (address or host name), which could lead to better diffs.
+        for host_a, host_b in host_pairs(self.scan_a.hosts, self.scan_b.hosts):
+            h_diff = HostDiff(host_a, host_b)
+            cost += h_diff.cost
             if h_diff.cost > 0 or verbose:
                 host = host_a or host_b
-                self.hosts.append(host)
-                self.host_diffs[host] = h_diff
+                self.output_host_diff(h_diff)
 
-        self.pre_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.pre_script_results, self.scan_b.pre_script_results)
-        self.post_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.post_script_results, self.scan_b.post_script_results)
+        post_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.post_script_results, self.scan_b.post_script_results)
+        self.output_post_scripts(post_script_result_diffs)
 
-    def print_text(self, f = sys.stdout):
-        """Print this diff in a human-readable text form."""
+        self.output_ending()
+
+        return cost
+
+class ScanDiffText(ScanDiff):
+    def __init__(self, scan_a, scan_b, f = sys.stdout):
+        ScanDiff.__init__(self, scan_a, scan_b, f)
+
+    def output_beginning(self):
         banner_a = format_banner(self.scan_a)
         banner_b = format_banner(self.scan_b)
         if banner_a != banner_b:
-            print >> f, u"-%s" % banner_a
-            print >> f, u"+%s" % banner_b
-        else:
-            print >> f, u" %s" % banner_a
+            print >> self.f, u"-%s" % banner_a
+            print >> self.f, u"+%s" % banner_b
+        elif verbose:
+            print >> self.f, u" %s" % banner_a
 
+    def output_pre_scripts(self, pre_script_result_diffs):
         print_script_result_diffs_text("Pre-scan script results",
             self.scan_a.pre_script_results, self.scan_b.pre_script_results,
-            self.pre_script_result_diffs)
-
-        for host in self.hosts:
-            print >> f
-
-            h_diff = self.host_diffs[host]
-            h_diff.print_text(f)
+            pre_script_result_diffs, self.f)
 
+    def output_post_scripts(self, post_script_result_diffs):
         print_script_result_diffs_text("Post-scan script results",
             self.scan_a.post_script_results, self.scan_b.post_script_results,
-            self.post_script_result_diffs)
+            post_script_result_diffs, self.f)
+
+    def output_host_diff(self, h_diff):
+        print >> self.f
+        h_diff.print_text(self.f)
+
+    def output_ending(self):
+        pass
+
+class ScanDiffXML(ScanDiff):
+    def __init__(self, scan_a, scan_b, f = sys.stdout):
+        ScanDiff.__init__(self, scan_a, scan_b, f)
 
-    def print_xml(self, f = sys.stdout):
         impl = xml.dom.minidom.getDOMImplementation()
-        document = impl.createDocument(None, u"nmapdiff", None)
-        root = document.documentElement
-        root.setAttribute(u"version", NDIFF_XML_VERSION)
-        scandiff_elem = document.createElement(u"scandiff")
-        root.appendChild(scandiff_elem)
+        self.document = impl.createDocument(None, None, None)
 
-        # prerule script changes.
-        if len(self.pre_script_result_diffs) > 0 or verbose:
-            prescript_elem = document.createElement(u"prescript")
-            scandiff_elem.appendChild(script_result_diffs_to_dom_fragment(
+        self.writer = XMLWriter(f)
+
+    def nmaprun_differs(self):
+        for attr in ("scanner", "version", "args", "start_date", "end_date"):
+            if getattr(self.scan_a, attr, None) != getattr(self.scan_b, attr, None):
+                return True
+        return False
+
+    def output_beginning(self):
+        self.writer.startDocument()
+        self.writer.startElement(u"nmapdiff", {u"version": NDIFF_XML_VERSION})
+        self.writer.startElement(u"scandiff", {})
+
+        if self.nmaprun_differs():
+            self.writer.frag_a(self.scan_a.nmaprun_to_dom_fragment(self.document))
+            self.writer.frag_b(self.scan_b.nmaprun_to_dom_fragment(self.document))
+        elif verbose:
+            self.writer.frag(self.scan_a.nmaprun_to_dom_fragment(self.document))
+
+    def output_pre_scripts(self, pre_script_result_diffs):
+        if len(pre_script_result_diffs) > 0 or verbose:
+            prescript_elem = self.document.createElement(u"prescript")
+            frag = script_result_diffs_to_dom_fragment(
                 prescript_elem, self.scan_a.pre_script_results,
-                self.scan_b.pre_script_results, self.pre_script_result_diffs,
-                document))
+                self.scan_b.pre_script_results, pre_script_result_diffs,
+                self.document)
+            self.writer.frag(frag)
+            frag.unlink()
 
-        for host in self.hosts:
-            h_diff = self.host_diffs[host]
-            frag = h_diff.to_dom_fragment(document)
-            scandiff_elem.appendChild(frag)
-
-        # postrule script changes.
-        if len(self.post_script_result_diffs) > 0 or verbose:
-            postscript_elem = document.createElement(u"postscript")
-            scandiff_elem.appendChild(script_result_diffs_to_dom_fragment(
+    def output_post_scripts(self, post_script_result_diffs):
+        if len(post_script_result_diffs) > 0 or verbose:
+            postscript_elem = self.document.createElement(u"postscript")
+            frag = script_result_diffs_to_dom_fragment(
                 postscript_elem, self.scan_a.post_script_results,
-                self.scan_b.post_script_results, self.post_script_result_diffs,
-                document))
+                self.scan_b.post_script_results, post_script_result_diffs,
+                self.document)
+            self.writer.frag(frag)
+            frag.unlink()
 
-        document.writexml(f, addindent = u"  ", newl = u"\n", encoding = "UTF-8")
-        document.unlink()
+    def output_host_diff(self, h_diff):
+        frag = h_diff.to_dom_fragment(self.document)
+        self.writer.frag(frag)
+        frag.unlink()
 
-    cost = property(lambda self: sum([hd.cost for hd in self.host_diffs.values()]))
+    def output_ending(self):
+        self.writer.endElement(u"scandiff")
+        self.writer.endElement(u"nmapdiff")
+        self.writer.endDocument()
 
 class HostDiff(object):
     """A diff of two Hosts. It contains the two hosts, variables describing what
@@ -1051,6 +1145,8 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
             if attrs.has_key(u"start"):
                 start_timestamp = int(attrs.get(u"start"))
                 self.scan.start_date = datetime.datetime.fromtimestamp(start_timestamp)
+            self.scan.scanner = attrs.get(u"scanner")
+            self.scan.args = attrs.get(u"args")
             self.scan.version = attrs.get(u"version")
         elif name == u"host":
             assert self.parent_element() == u"nmaprun"
@@ -1186,6 +1282,27 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
             self.current_port.script_results.sort()
             self.current_port = None
 
+class XMLWriter (xml.sax.saxutils.XMLGenerator):
+    def __init__(self, f):
+        xml.sax.saxutils.XMLGenerator.__init__(self, f, "utf-8")
+        self.f = f
+
+    def frag(self, frag):
+        for node in frag.childNodes:
+            node.writexml(self.f, newl = u"\n")
+
+    def frag_a(self, frag):
+        self.startElement(u"a", {})
+        for node in frag.childNodes:
+            node.writexml(self.f, newl = u"\n")
+        self.endElement(u"a")
+
+    def frag_b(self, frag):
+        self.startElement(u"b", {})
+        for node in frag.childNodes:
+            node.writexml(self.f, newl = u"\n")
+        self.endElement(u"b")
+
 def usage():
     print u"""\
 Usage: %s [option] FILE1 FILE2
@@ -1249,14 +1366,13 @@ def main():
         print >> sys.stderr, u"Can't open file: %s" % str(e)
         sys.exit(EXIT_ERROR)
 
-    diff = ScanDiff(scan_a, scan_b)
-
     if output_format == "text":
-        diff.print_text()
+        diff = ScanDiffText(scan_a, scan_b)
     elif output_format == "xml":
-        diff.print_xml()
+        diff = ScanDiffXML(scan_a, scan_b)
+    cost = diff.output()
 
-    if diff.cost == 0:
+    if cost == 0:
         return EXIT_EQUAL
     else:
         return EXIT_DIFFERENT
