@@ -8,7 +8,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -18,11 +18,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -36,19 +37,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -72,15 +74,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -100,9 +103,6 @@
 #include "utils.h"
 #include "utils_net.h"
 #include "output.h"
-#include "ICMPv4Header.h"
-#include "ARPHeader.h"
-
 
 extern NpingOps o;
 
@@ -252,6 +252,8 @@ char errstr[256];
   {"no-crypto", no_argument, 0, 0},
   {"nc", no_argument, 0, 0},
   {"once", no_argument, 0, 0},
+  {"safe-payloads", no_argument, 0, 0},
+  {"include-payloads", no_argument, 0, 0},
 
   /* Timing and performance */
   {"delay", required_argument, 0, 0},
@@ -269,6 +271,7 @@ char errstr[256];
   {"send-ip", no_argument, 0, 0},
   {"bpf-filter", required_argument, 0, 0},
   {"filter", required_argument, 0, 0},
+  {"nsock-engine", required_argument, 0, 0},
   {"no-capture", no_argument, 0, 'N'},
   {"hide-sent", no_argument, 0, 'H'},
     
@@ -413,7 +416,7 @@ char errstr[256];
                 if( aux8 & 0x10 )
                     o.setFlagTCP( FLAG_ACK );
                 if( aux8 & 0x08 )
-                    o.setFlagTCP( FLAG_PUSH );
+                    o.setFlagTCP( FLAG_PSH );
                 if( aux8 & 0x04 )
                     o.setFlagTCP( FLAG_RST );
                 if( aux8 & 0x02 )
@@ -432,7 +435,7 @@ char errstr[256];
                 else if(!strncasecmp((optarg+f), "ECE",3)){ o.setFlagTCP(FLAG_ECN);  }
                 else if(!strncasecmp((optarg+f), "URG",3)){ o.setFlagTCP(FLAG_URG);  }
                 else if(!strncasecmp((optarg+f), "ACK",3)){ o.setFlagTCP(FLAG_ACK);  }
-                else if(!strncasecmp((optarg+f), "PSH",3)){ o.setFlagTCP(FLAG_PUSH);  }
+                else if(!strncasecmp((optarg+f), "PSH",3)){ o.setFlagTCP(FLAG_PSH);  }
                 else if(!strncasecmp((optarg+f), "RST",3)){ o.setFlagTCP(FLAG_RST);  }
                 else if(!strncasecmp((optarg+f), "SYN",3)){ o.setFlagTCP(FLAG_SYN);  }
                 else if(!strncasecmp((optarg+f), "FIN",3)){ o.setFlagTCP(FLAG_FIN);  }
@@ -457,7 +460,7 @@ char errstr[256];
                 else if(!strcasecmp(optarg, "ECE")){ o.setFlagTCP(FLAG_ECN); flag3_ok=true; }
                 else if(!strcasecmp(optarg, "URG")){ o.setFlagTCP(FLAG_URG); flag3_ok=true; }
                 else if(!strcasecmp(optarg, "ACK")){ o.setFlagTCP(FLAG_ACK); flag3_ok=true; }
-                else if(!strcasecmp(optarg, "PSH")){ o.setFlagTCP(FLAG_PUSH); flag3_ok=true; }
+                else if(!strcasecmp(optarg, "PSH")){ o.setFlagTCP(FLAG_PSH); flag3_ok=true; }
                 else if(!strcasecmp(optarg, "RST")){ o.setFlagTCP(FLAG_RST); flag3_ok=true; }
                 else if(!strcasecmp(optarg, "SYN")){ o.setFlagTCP(FLAG_SYN); flag3_ok=true; }
                 else if(!strcasecmp(optarg, "FIN")){ o.setFlagTCP(FLAG_FIN); flag3_ok=true; }
@@ -480,7 +483,7 @@ char errstr[256];
                         case 'E': case 'e': o.setFlagTCP(FLAG_ECN); break;
                         case 'U': case 'u': o.setFlagTCP(FLAG_URG); break;
                         case 'A': case 'a': o.setFlagTCP(FLAG_ACK); break;
-                        case 'P': case 'p': o.setFlagTCP(FLAG_PUSH); break;
+                        case 'P': case 'p': o.setFlagTCP(FLAG_PSH); break;
                         case 'R': case 'r': o.setFlagTCP(FLAG_RST); break;
                         case 'S': case 's': o.setFlagTCP(FLAG_SYN); break;
                         case 'F': case 'f': o.setFlagTCP(FLAG_FIN); break;
@@ -902,6 +905,10 @@ char errstr[256];
     } else if (optcmp(long_options[option_index].name, "no-crypto")==0 ||
                optcmp(long_options[option_index].name, "nc")==0 ){
         o.doCrypto(false);
+    } else if (optcmp(long_options[option_index].name, "safe-payloads")==0 ){
+        o.echoPayload(false);
+    } else if (optcmp(long_options[option_index].name, "include-payloads")==0 ){
+        o.echoPayload(true);
 
 
 /* TIMING AND PERFORMANCE OPTIONS ********************************************/
@@ -948,7 +955,9 @@ char errstr[256];
         o.setBPFFilterSpec( optarg );
         if( o.issetDisablePacketCapture() && o.disablePacketCapture()==true )
             outError(QT_2, "Warning: There is no point on specifying a BPF filter if you disable packet capture. BPF filter will be ignored.");
-        
+    } else if (optcmp(long_options[option_index].name, "nsock-engine") == 0){
+        nsock_set_default_engine(optarg);
+
     /* Output Options */
     } else if (optcmp(long_options[option_index].name, "quiet") == 0 ){
             o.setVerbosity(-4);
@@ -1283,6 +1292,7 @@ void ArgParser::printUsage(void){
 "  --echo-port <port>               : Use custom <port> to listen or connect.\n"
 "  --no-crypto                      : Disable encryption and authentication.\n"
 "  --once                           : Stop the server after one connection.\n"
+"  --safe-payloads                  : Erase application data in echoed packets.\n"
 "TIMING AND PERFORMANCE:\n"
  "  Options which take <time> are in seconds, or append 'ms' (milliseconds),\n"
 "  's' (seconds), 'm' (minutes), or 'h' (hours) to the value (e.g. 30m, 0.25h).\n"
