@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -17,11 +17,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -35,19 +36,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -71,15 +73,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -114,8 +117,8 @@ NpingTarget::NpingTarget() {
 
 /** Initializes object attributes  */
 void NpingTarget::Initialize() {
-  memset(this->devname, 0, 32);
-  memset(this->devfullname, 0, 32);
+  memset(this->devname, 0, sizeof(this->devname));
+  memset(this->devfullname, 0, sizeof(this->devfullname));
   dev_type=devt_other;
   directly_connected = -1;
   distance = -1;
@@ -786,7 +789,7 @@ bool NpingTarget::determineNextHopMACAddress() {
   }
 
   /* First, let us check the ARP cache ... */
-  if (arp_cache_get(&targetss, mac)) {
+  if (mac_cache_get(&targetss, mac)) {
     this->setNextHopMACAddress(mac);
     return true;
   }
@@ -796,7 +799,7 @@ bool NpingTarget::determineNextHopMACAddress() {
   a = arp_open();
   addr_ston((sockaddr *)&targetss, &ae.arp_pa);
   if (arp_get(a, &ae) == 0) {
-    arp_cache_set(&targetss, ae.arp_ha.addr_eth.data);
+    mac_cache_set(&targetss, ae.arp_ha.addr_eth.data);
     this->setNextHopMACAddress(ae.arp_ha.addr_eth.data);
     arp_close(a);
     outPrint(DBG_3,"    > Success: Entry found [%s]", this->getNextHopMACStr() );
@@ -813,7 +816,7 @@ bool NpingTarget::determineNextHopMACAddress() {
     outPrint(DBG_3,"    > Sending ARP request using spoofed IP %s...", this->getSpoofedSourceIPStr() );
       this->getSpoofedSourceSockAddr(&srcss, NULL);
       if (doArp(this->getDeviceName(), this->getSrcMACAddress(), &srcss, &targetss, mac, NULL)) {
-        arp_cache_set(&targetss, mac);
+        mac_cache_set(&targetss, mac);
         this->setNextHopMACAddress(mac);
         outPrint(DBG_4,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
         return true;
@@ -825,7 +828,7 @@ bool NpingTarget::determineNextHopMACAddress() {
   outPrint(DBG_4,"    > Sending ARP request using our real IP %s...", this->getSourceIPStr() );
   this->getSourceSockAddr(&srcss, NULL);
   if (doArp(this->getDeviceName(), this->getSrcMACAddress(), &srcss, &targetss, mac, NULL)) {
-    arp_cache_set(&targetss, mac);
+    mac_cache_set(&targetss, mac);
     this->setNextHopMACAddress(mac);
     outPrint(DBG_3,"    > Success: 1 ARP response received [%s]", this->getNextHopMACStr() );
     return true;
