@@ -355,7 +355,6 @@ static void trace (nsock_iod nsiod, const char *message, const char *dir)
   {
     if (!nsi_is_pcap(nsiod))
     {
-      int status;
       int protocol;
       int af;
       char ipstring_local[INET6_ADDRSTRLEN];
@@ -363,7 +362,7 @@ static void trace (nsock_iod nsiod, const char *message, const char *dir)
       struct sockaddr_storage local;
       struct sockaddr_storage remote;
 
-      status = nsi_getlastcommunicationinfo(nsiod, &protocol, &af,
+      nsi_getlastcommunicationinfo(nsiod, &protocol, &af,
           (sockaddr *) &local, (sockaddr *) &remote, sizeof(sockaddr_storage));
       log_write(LOG_STDOUT, "%s: %s %s:%d %s %s:%d | %s\n",
           SCRIPT_ENGINE,
@@ -576,7 +575,8 @@ static int l_connect (lua_State *L)
       break;
   }
 
-  freeaddrinfo(dest);
+  if (dest != NULL)
+    freeaddrinfo(dest);
   return yield(L, nu, "CONNECT", TO, 0, NULL);
 }
 
@@ -733,16 +733,16 @@ static int l_receive_buf (lua_State *L)
 static int l_get_info (lua_State *L)
 {
   nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
-  int status;
   int protocol;                                  // tcp or udp
   int af;                                        // address family
-  struct sockaddr local;
-  struct sockaddr remote;
+  struct sockaddr_storage local;
+  struct sockaddr_storage remote;
   char *ipstring_local = (char *) lua_newuserdata(L, sizeof(char) * INET6_ADDRSTRLEN);
   char *ipstring_remote = (char *) lua_newuserdata(L, sizeof(char) * INET6_ADDRSTRLEN);
 
-  status = nsi_getlastcommunicationinfo(nu->nsiod, &protocol, &af,
-      &local, &remote, sizeof(sockaddr));
+  nsi_getlastcommunicationinfo(nu->nsiod, &protocol, &af,
+      (struct sockaddr*)&local, (struct sockaddr*)&remote,
+      sizeof(struct sockaddr_storage));
 
   lua_pushboolean(L, true);
   lua_pushstring(L, inet_ntop_both(af, &local, ipstring_local));
@@ -756,7 +756,7 @@ static int l_set_timeout (lua_State *L)
 {
   nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
   nu->timeout = luaL_checkint(L, 2);
-  if ((int) nu->timeout < -1)
+  if ((int) nu->timeout < -1) /* -1 is no timeout */
     return luaL_error(L, "Negative timeout: %d", nu->timeout);
   return success(L);
 }
@@ -789,7 +789,7 @@ SSL *nse_nsock_get_ssl (lua_State *L)
 {
   nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
 
-  if (!nsi_checkssl(nu->nsiod))
+  if (nu->nsiod == NULL || !nsi_checkssl(nu->nsiod))
     luaL_argerror(L, 1, "not a SSL socket");
 
   return (SSL *) nsi_getssl(nu->nsiod);
