@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -14,11 +14,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -26,31 +27,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -68,15 +70,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -89,6 +92,8 @@
 
 /* $Id:$ */
 
+#include "nbase.h"
+
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
@@ -102,7 +107,6 @@ extern "C"
 {
 #include "lua.h"
 #include "lauxlib.h"
-void *safe_realloc(void *, size_t);
 }
 
 #include "nse_nsock.h"
@@ -161,14 +165,16 @@ static void obj_to_key(lua_State *L, const ASN1_OBJECT *obj)
 
   nid = OBJ_obj2nid(obj);
   if (nid == NID_undef) {
-    size_t size = 0;
-    char *buf = NULL;
+    size_t size = 1;
+    char *buf = (char *) lua_newuserdata(L, size);
     const char *p, *q;
     int i, n;
 
     while ((n = OBJ_obj2txt(buf, size, obj, 1)) < 0 || (unsigned) n >= size) {
-      size = size * 2 + 1;
-      buf = (char *) safe_realloc(buf, size);
+      size = size * 2;
+      buf = (char *) lua_newuserdata(L, size);
+      memcpy(lua_touserdata(L, -1), lua_touserdata(L, -2), lua_objlen(L, -2));
+      lua_replace(L, -2);
     }
 
     lua_newtable(L);
@@ -184,7 +190,7 @@ static void obj_to_key(lua_State *L, const ASN1_OBJECT *obj)
       lua_rawseti(L, -2, i++);
       p = q + 1;
     }
-    free(buf);
+    lua_replace(L, -2); /* replace userdata with table */
   } else {
     lua_pushstring(L, OBJ_nid2ln(nid));
   }
@@ -243,7 +249,7 @@ static int parse_int(const unsigned char *s, size_t len)
   return (int) v;
 }
 
-/* This is a helper function for asn1_time_to_obj. It parses a tectual ASN1_TIME
+/* This is a helper function for asn1_time_to_obj. It parses a textual ASN1_TIME
    value and stores the time in the given struct tm. It returns 0 on success and
    -1 on a parse error. */
 static int time_to_tm(const ASN1_TIME *t, struct tm *result)
@@ -320,7 +326,7 @@ static void tm_to_table(lua_State *L, const struct tm *tm)
   /* Omit tm_wday and tm_yday. */
 }
 
-/* This is a helper functino for x509_validity_to_table. It takes teh given
+/* This is a helper function for x509_validity_to_table. It takes teh given
    ASN1_TIME and covnerts it to a value on the stack, which is one of
      nil, if the time is NULL;
      a date table, if the date can be parsed; and
@@ -370,12 +376,33 @@ static void cert_pem_to_string(lua_State *L, X509 *cert)
   BIO_vfree(bio);
 }
 
+/* This is a helper function for l_get_ssl_certificate. It converts the
+   public-key type to a string. */
+static const char *pkey_type_to_string(int type)
+{
+  switch (type) {
+  case EVP_PKEY_RSA:
+    return "rsa";
+  case EVP_PKEY_DSA:
+    return "dsa";
+  case EVP_PKEY_DH:
+    return "dh";
+#ifdef EVP_PKEY_EC
+  case EVP_PKEY_EC:
+    return "ec";
+#endif
+  default:
+    return "unknown";
+  }
+}
+
 int l_get_ssl_certificate(lua_State *L)
 {
   SSL *ssl;
   struct cert_userdata *udata;
   X509 *cert;
   X509_NAME *subject, *issuer;
+  EVP_PKEY *pubkey;
 
   ssl = nse_nsock_get_ssl(L);
   cert = SSL_get_peer_certificate(ssl);
@@ -407,6 +434,15 @@ int l_get_ssl_certificate(lua_State *L)
   cert_pem_to_string(L, cert);
   lua_setfield(L, -2, "pem");
 
+  pubkey = X509_get_pubkey(cert);
+  lua_newtable(L);
+  lua_pushstring(L, pkey_type_to_string(pubkey->type));
+  lua_setfield(L, -2, "type");
+  lua_pushnumber(L, EVP_PKEY_bits(pubkey));
+  lua_setfield(L, -2, "bits");
+  lua_setfield(L, -2, "pubkey");
+  EVP_PKEY_free(pubkey);
+
   /* At this point the certificate-specific table of attributes is at the top of
      the stack. We give it a metatable with an __index entry that points into
      the global shared table of certificate functions. */
