@@ -4,7 +4,7 @@
 -- Because HTTP has so many uses, there are a number of interfaces to this library.
 -- The most obvious and common ones are simply <code>get</code>, <code>post</code>,
 -- and <code>head</code>; or, if more control is required, <code>generic_request</code>
--- can be used. Thse functions do what one would expect. The <code>get_url</code> 
+-- can be used. These functions do what one would expect. The <code>get_url</code> 
 -- helper function can be used to parse and retrieve a full URL. 
 --
 -- These functions return a table of values, including:
@@ -12,12 +12,12 @@
 -- * <code>header</code> - An associative array representing the header. Keys are all lowercase, and standard headers, such as 'date', 'content-length', etc. will typically be present. 
 -- * <code>rawheader</code> - A numbered array of the headers, exactly as the server sent them. While header['content-type'] might be 'text/html', rawheader[3] might be 'Content-type: text/html'.
 -- * <code>cookies</code> - A numbered array of the cookies the server sent. Each cookie is a table with the following keys: <code>name</code>, <code>value</code>, <code>path</code>, <code>domain</code>, and <code>expires</code>. 
--- * <code>body</code> - The full body, as retunred by the server. 
+-- * <code>body</code> - The full body, as returned by the server. 
 --
--- If a script is planning on making a lot of requests, the pipeling functions can
+-- If a script is planning on making a lot of requests, the pipelining functions can
 -- be helpful. <code>pipeline_add</code> queues requests in a table, and
 -- <code>pipeline</code> performs the requests, returning the results as an array,
--- with the respones in the same order as the queries were added. As a simple example:
+-- with the responses in the same order as the queries were added. As a simple example:
 --<code>
 --	-- Start by defining the 'all' variable as nil
 --	local all = nil
@@ -39,7 +39,7 @@
 -- a page exists. The <code>identify_404</code> function will try several URLs on the 
 -- server to determine what the server's 404 pages look like. It will attempt to identify
 -- customized 404 pages that may not return the actual status code 404. If successful, 
--- the function <code>page_exists</code> can then be used to determine whether no not
+-- the function <code>page_exists</code> can then be used to determine whether or not
 -- a page existed. 
 --
 -- Some other miscellaneous functions that can come in handy are <code>response_contains</code>,
@@ -69,6 +69,18 @@
 -- * <code>bypass_cache</code>: Do not perform a lookup in the local HTTP cache.
 -- * <code>no_cache</code>: Do not save the result of this request to the local HTTP cache.
 -- * <code>no_cache_body</code>: Do not save the body of the response to the local HTTP cache.
+-- * <code>redirect_ok</code>: Closure that overrides the default redirect_ok used to validate whether to follow HTTP redirects or not. False, if no HTTP redirects should be followed.
+--   The following example shows how to write a custom closure that follows 5 consecutive redirects:
+--   <code>
+--   redirect_ok = function(host,port)
+--     local c = 5
+--     return function(url)
+--       if ( c==0 ) then return false end
+--       c = c - 1
+--       return true
+--     end
+--   end
+--   </code>
 --  
 -- @args http-max-cache-size The maximum memory size (in bytes) of the cache.
 --
@@ -81,6 +93,10 @@
 -- pipelined (ie, sent in a single request). This can be set low to make
 -- debugging easier, or it can be set high to test how a server reacts (its
 -- chosen max is ignored).
+--
+-- TODO
+-- Implement cache system for http pipelines
+--
 
 local coroutine = require "coroutine";
 local table = require "table";
@@ -97,6 +113,7 @@ module(... or "http",package.seeall)
 local have_ssl = (nmap.have_ssl() and pcall(require, "openssl"))
 
 local USER_AGENT = stdnse.get_script_args('http.useragent') or "Mozilla/5.0 (compatible; Nmap Scripting Engine; http://nmap.org/book/nse.html)"
+local MAX_REDIRECT_COUNT = 5
 
 -- Recursively copy a table.
 -- Only recurs when a value is a table, other values are copied by assignment.
@@ -266,8 +283,8 @@ local function validate_options(options)
       end
     elseif(key == 'cookies') then
       if(type(value) == 'table') then
-        for cookie in pairs(value) do
-          for cookie_key, cookie_value in pairs(value) do
+        for _, cookie in ipairs(value) do
+          for cookie_key, cookie_value in pairs(cookie) do
             if(cookie_key == 'name') then
               if(type(cookie_value) ~= 'string') then
                 stdnse.print_debug(1, "http: options.cookies[i].name should be a string")
@@ -283,6 +300,11 @@ local function validate_options(options)
                 stdnse.print_debug(1, "http: options.cookies[i].path should be a string")
                 bad = true
               end
+            elseif(cookie_key == 'expires') then
+              if(type(cookie_value) ~= 'string') then
+                stdnse.print_debug(1, "http: options.cookies[i].expires should be a string")
+                bad = true
+              end
             else
               stdnse.print_debug(1, "http: Unknown field in cookie table: %s", cookie_key)
               bad = true
@@ -308,6 +330,11 @@ local function validate_options(options)
         stdnse.print_debug(1, "http: options.bypass_cache, options.no_cache, and options.no_cache_body must be boolean values")
         bad = true
       end
+    elseif(key == 'redirect_ok') then
+      if(type(value)~= 'function' and type(value)~='boolean') then
+        stdnse.print_debug(1, "http: options.redirect_ok must be a function or boolean")
+        bad = true
+      end
     else
       stdnse.print_debug(1, "http: Unknown key in the options table: %s", key)
     end
@@ -484,7 +511,9 @@ local function recv_chunked(s, partial)
 
     line, partial = recv_line(s, partial)
     if not line then
-      return nil, string.format("Didn't find CRLF after chunk-data.")
+	  -- this warning message was initially an error but was adapted
+	  -- to support broken servers, such as the Citrix XML Service
+      stdnse.print_debug(2, "Didn't find CRLF after chunk-data.")
     elseif not string.match(line, "^\r?\n") then
       return nil, string.format("Didn't find CRLF after chunk-data; got %q.", line)
     end
@@ -731,7 +760,7 @@ local function parse_set_cookie(s)
       else
         value = true
       end
-      cookie[name] = value
+      cookie[name:lower()] = value
       pos = skip_space(s, pos)
     end
 
@@ -820,13 +849,13 @@ local function getPipelineMax(response)
   local pipeline = stdnse.get_script_args({'http.pipeline', 'pipeline'})
 
   if(pipeline) then
-    return tonumber(nmap.registry.args.pipeline)
+    return tonumber(pipeline)
   end
 
   if response then
     if response.header and response.header.connection ~= "close" then
       if response.header["keep-alive"] then
-        local max = string.match( response.header["keep-alive"], "max\=(%d*)")
+        local max = string.match( response.header["keep-alive"], "max=(%d*)")
         if(max == nil) then
           return 40
         end
@@ -1072,11 +1101,12 @@ local function build_request(host, port, method, path, options)
 
   -- Add any other header fields into the local copy.
   table_augment(mod_options, options)
-
-  local request_line = string.format("%s %s HTTP/1.1", method, path)
+  -- We concat this string manually to allow null bytes in requests
+  local request_line = method.." "..path.." HTTP/1.1"
   local header = {}
   for name, value in pairs(mod_options.header) do
-    header[#header + 1] = string.format("%s: %s", name, value)
+    -- we concat this string manually to allow null bytes in requests
+    header[#header + 1] = name..": "..value
   end
 
   return request_line .. "\r\n" .. stdnse.strjoin("\r\n", header) .. "\r\n\r\n" .. (body or "")
@@ -1101,7 +1131,7 @@ local function request(host, port, data, options)
     return nil
   end
   local method
-  local header, partial
+  local header
   local response
 
   options = options or {}
@@ -1114,11 +1144,10 @@ local function request(host, port, data, options)
   end
 
   local error_response = {status=nil,["status-line"]=nil,header={},body=""}
-  local socket
 
   method = string.match(data, "^(%S+)")
 
-  socket, partial = comm.tryssl(host, port, data, { timeout = options.timeout })
+  local socket, partial, opts = comm.tryssl(host, port, data, { timeout = options.timeout })
 
   if not socket then
     return error_response
@@ -1136,6 +1165,9 @@ local function request(host, port, data, options)
 
   socket:close()
 
+  -- if SSL was used to retrieve the URL mark this in the response
+  response.ssl = ( opts == 'ssl' )
+
   return response
 end
 
@@ -1161,10 +1193,179 @@ function generic_request(host, port, method, path, options)
   return request(host, port, build_request(host, port, method, path, options), options)
 end
 
+---Uploads a file using the PUT method and returns a result table. This is a simple wrapper
+-- around <code>generic_request</code>
+--
+-- @param host The host to connect to.
+-- @param port The port to connect to.
+-- @param path The path to retrieve.
+-- @param options [optional] A table that lets the caller control socket timeouts, HTTP headers, and other parameters. For full documentation, see the module documentation (above). 
+-- @param putdata The contents of the file to upload
+-- @return <code>nil</code> if an error occurs; otherwise, a table as described in the module documentation. 
+-- @see http.generic_request
+function put(host, port, path, options, putdata)
+  if(not(validate_options(options))) then
+    return nil
+  end
+  if ( not(putdata) ) then
+	return nil
+  end
+  local mod_options = {
+    content = putdata,
+  }
+  table_augment(mod_options, options or {})
+  return generic_request(host, port, "PUT", path, mod_options)
+end
+
+-- Check if the given URL is okay to redirect to. Return a table with keys
+-- "host", "port", and "path" if okay, nil otherwise.
+-- @param url table as returned by url.parse
+-- @param host table as received by the action function
+-- @param port table as received by the action function
+-- @return loc table containing the new location
+function redirect_ok(host, port)
+
+  -- A battery of tests a URL is subjected to in order to decide if it may be
+  -- redirected to. They incrementally fill in loc.host, loc.port, and loc.path.
+  local rules = {
+
+	-- Check if there's any credentials in the url
+    function (url, host, port)
+      -- bail if userinfo is present
+      return ( url.userinfo and false ) or true
+    end,
+
+    -- Check if the location is within the domain or host
+    function (url, host, port)
+      local hostname = stdnse.get_hostname(host)
+      if ( hostname == host.ip and host.ip == url.host.ip ) then
+        return true
+      end
+      local domain = hostname:match("^[^%.]-%.(.*)") or hostname
+      local match = ("^.*%s$"):format(domain)
+      if ( url.host:match(match) ) then
+        return true
+      end
+      return false
+    end,
+
+    -- Check whether the new location has the same port number
+    function (url, host, port)
+      -- port fixup, adds default ports 80 and 443 in case no url.port was
+      -- defined, we do this based on the url scheme
+      local url_port = url.port
+      if ( not(url_port) ) then
+        if ( url.scheme == "http" ) then
+          url_port = 80
+        elseif( url.scheme == "https" ) then
+          url_port = 443
+        end
+      end
+      if (not url_port) or tonumber(url_port) == port.number then
+        return true
+      end
+      return false
+    end,
+
+    -- Check whether the url.scheme matches the port.service
+    function (url, host, port)
+      -- if url.scheme is present then it must match the scanned port
+      if url.scheme and url.port then return true end
+      if url.scheme and url.scheme ~= port.service then return false end
+      return true
+    end,
+
+    -- make sure we're actually being redirected somewhere and not to the same url
+    function (url, host, port)
+      -- path cannot be unchanged unless host has changed
+      -- loc.path must be set if returning true
+      if ( not url.path or url.path == "/" ) and url.host == ( host.targetname or host.ip) then return false end
+      if not url.path then return true end
+      return true
+    end,
+  }
+
+  local counter = MAX_REDIRECT_COUNT
+  -- convert a numeric port to a table
+  if ( "number" == type(port) ) then
+    port = { number = port }
+  end
+  return function(url)
+    if ( counter == 0 ) then return false end
+    counter = counter - 1
+    for i, rule in ipairs( rules ) do
+	  if ( not(rule( url, host, port )) ) then
+        --stdnse.print_debug("Rule failed: %d", i)
+        return false
+      end
+    end
+    return true
+  end
+end
+
+-- Handles a HTTP redirect
+-- @param host table as received by the script action function
+-- @param port table as received by the script action function
+-- @param path string
+-- @param response table as returned by http.get or http.head
+-- @return url table as returned by <code>url.parse</code> or nil if there's no
+--         redirect taking place
+local function parse_redirect(host, port, path, response)
+  if ( not(tostring(response.status):match("^30[127]$")) or 
+       not(response.header) or
+       not(response.header.location) ) then
+    return nil
+  end
+  port = ( "number" == type(port) ) and { number = port } or port
+  local u = url.parse(response.header.location)
+  if ( not(u.host) and not(u.scheme) ) then
+    -- we're dealing with a relative url
+    u.host, u.port = stdnse.get_hostname(host), port.number
+    u.path = ((u.path:sub(1,1) == "/" and "" ) or "/" ) .. u.path -- ensuring leading slash
+  end
+  if ( u.query ) then
+    u.path = ("%s?%s"):format( u.path, u.query )
+  end
+  -- do port fixup
+  if ( not(u.port) ) then
+    if ( u.scheme == "http" ) then u.port = 80 end
+    if ( u.scheme == "https") then u.port = 443 end
+  end
+  return u
+end
+
+-- Retrieves the correct function to use to validate HTTP redirects
+-- @param host table as received by the action function
+-- @param port table as received by the action function
+-- @param options table as passed to http.get or http.head
+-- @return redirect_ok function used to validate HTTP redirects
+local function get_redirect_ok(host, port, options)
+  if ( options ) then
+    if ( options.redirect_ok == false ) then
+      return function() return false end
+    elseif( "function" == type(options.redirect_ok) ) then
+	  return options.redirect_ok(host, port)
+	else
+      return redirect_ok(host, port)
+    end
+  else
+    return redirect_ok(host, port)
+  end
+end
+
 ---Fetches a resource with a GET request and returns the result as a table. This is a simple
--- wraper around <code>generic_request</code>, with the added benefit of having local caching. 
--- This caching can be controlled in the <code>options</code> array, see module documentation 
--- for more information. 
+-- wraper around <code>generic_request</code>, with the added benefit of having local caching
+-- and support for HTTP redirects. Redirects are followed only if they pass all the
+-- validation rules of the redirect_ok function. This function may be overrided by supplying
+-- a custom function in the <code>redirect_ok</code> field of the options array. The default
+-- function redirects the request if the destination is:
+-- * Within the same host or domain
+-- * Has the same port number
+-- * Stays within the current scheme
+-- * Does not exceed <code>MAX_REDIRECT_COUNT</code> count of redirects
+-- 
+-- Caching and redirects can be controlled in the <code>options</code> array, see module
+-- documentation for more information. 
 --
 -- @param host The host to connect to.
 -- @param port The port to connect to.
@@ -1176,11 +1377,23 @@ function get(host, port, path, options)
   if(not(validate_options(options))) then
     return nil
   end
-  local response, state = lookup_cache("GET", host, port, path, options);
-  if response == nil then
-    response = generic_request(host, port, "GET", path, options)
-    insert_cache(state, response);
-  end
+  local redir_check = get_redirect_ok(host, port, options)
+  local response, state, location
+  local u = { host = host, port = port, path = path }
+  repeat
+    response, state = lookup_cache("GET", u.host, u.port, u.path, options);
+    if ( response == nil ) then
+      response = generic_request(u.host, u.port, "GET", u.path, options)
+      insert_cache(state, response);
+    end
+    u = parse_redirect(host, port, path, response)
+    if ( not(u) ) then
+      break
+    end
+    location = location or {}
+    table.insert(location, response.header.location)
+  until( not(redir_check(u)) )
+  response.location = location
   return response
 end
 
@@ -1218,7 +1431,19 @@ function get_url( u, options )
 end
 
 ---Fetches a resource with a HEAD request. Like <code>get</code>, this is a simple
--- wrapper around <code>generic_request</code> with response caching. 
+-- wrapper around <code>generic_request</code> with response caching. This function
+-- also has support for HTTP redirects. Redirects are followed only if they pass
+-- all the validation rules of the redirect_ok function. This function may be
+-- overrided by supplying a custom function in the <code>redirect_ok</code> field
+-- of the options array. The default function redirects the request if the
+-- destination is:
+-- * Within the same host or domain
+-- * Has the same port number
+-- * Stays within the current scheme
+-- * Does not exceed <code>MAX_REDIRECT_COUNT</code> count of redirects
+-- 
+-- Caching and redirects can be controlled in the <code>options</code> array,
+-- see module documentation for more information.
 --
 -- @param host The host to connect to.
 -- @param port The port to connect to.
@@ -1230,12 +1455,24 @@ function head(host, port, path, options)
   if(not(validate_options(options))) then
     return nil
   end
-  local response, state = lookup_cache("HEAD", host, port, path, options);
-  if response == nil then
-    response = generic_request(host, port, "HEAD", path, options)
-    insert_cache(state, response);
-  end
-  return response;
+  local redir_check = get_redirect_ok(host, port, options)
+  local response, state, location
+  local u = { host = host, port = port, path = path }
+  repeat
+    response, state = lookup_cache("HEAD", host, port, path, options);
+    if response == nil then
+      response = generic_request(host, port, "HEAD", path, options)
+      insert_cache(state, response);
+    end
+    u = parse_redirect(host, port, path, response)
+    if ( not(u) ) then
+      break
+    end
+    location = location or {}
+    table.insert(location, response.header.location)
+  until( not(redir_check(u)) )
+  response.location = location
+  return response
 end
 
 ---Fetches a resource with a POST request. Like <code>get</code>, this is a simple
@@ -1456,12 +1693,12 @@ local function read_quoted_string(s, pos)
   end
   pos = pos + 1
   pos = skip_space(s, pos)
-  while pos <= string.len(s) and string.sub(s, pos, pos) ~= "\"" do
+  while pos <= #s and string.sub(s, pos, pos) ~= "\"" do
     local c
 
     c = string.sub(s, pos, pos)
     if c == "\\" then
-      if pos < string.len(s) then
+      if pos < #s then
         pos = pos + 1
         c = string.sub(s, pos, pos)
       else
@@ -1472,7 +1709,7 @@ local function read_quoted_string(s, pos)
     chars[#chars + 1] = c
     pos = pos + 1
   end
-  if pos > string.len(s) or string.sub(s, pos, pos) ~= "\"" then
+  if pos > #s or string.sub(s, pos, pos) ~= "\"" then
     return nil
   end
 
@@ -1556,7 +1793,7 @@ local function read_auth_challenge(s, pos)
 
   params = {}
   pos = skip_space(s, pos)
-  while pos < string.len(s) do
+  while pos < #s do
     local name, val
     local tmp_pos
 
@@ -1569,7 +1806,8 @@ local function read_auth_challenge(s, pos)
     tmp_pos = pos
     tmp_pos, name = read_token(s, tmp_pos)
     if not name then
-      return nil
+      pos = skip_space(s, pos + 1)
+      return pos, { scheme = scheme, params = nil }
     end
     tmp_pos = skip_space(s, tmp_pos)
     if string.sub(s, tmp_pos, tmp_pos) ~= "=" then
@@ -1590,7 +1828,7 @@ local function read_auth_challenge(s, pos)
     pos = skip_space(s, pos)
     if string.sub(s, pos, pos) == "," then
       pos = skip_space(s, pos + 1)
-      if pos > string.len(s) then
+      if pos > #s then
         return nil
       end
     end
@@ -1610,7 +1848,7 @@ function parse_www_authenticate(s)
   local pos
 
   pos = 1
-  while pos <= string.len(s) do
+  while pos <= #s do
     local challenge
 
     pos, challenge = read_auth_challenge(s, pos)
@@ -1691,7 +1929,7 @@ function can_use_head(host, port, result_404, path)
 
     if data.status and data.status == 200 and data.header then
       -- check that a body wasn't returned
-      if string.len(data.body) > 0 then
+      if #data.body > 0 then
         stdnse.print_debug(1, "HTTP: Warning: Host returned data when performing HEAD.")
         return false
       end
@@ -1726,7 +1964,7 @@ end
 -- the user.
 --
 -- @param body The body of the page.
-local function clean_404(body)
+function clean_404(body)
   -- Remove anything that looks like time
   body = string.gsub(body, '%d?%d:%d%d:%d%d', "")
   body = string.gsub(body, '%d%d:%d%d', "")
@@ -1887,7 +2125,7 @@ function page_exists(data, result_404, known_404, page, displayall)
     if(data.status == 200) then
       if(result_404 == 200) then
         -- If the 404 response is also "200", deal with it (check if the body matches)
-        if(string.len(data.body) == 0) then
+        if(#data.body == 0) then
           -- I observed one server that returned a blank string instead of an error, on some occasions
           stdnse.print_debug(1, "HTTP: Page returned a totally empty body; page likely doesn't exist")
           return false
